unit ODBFForm;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, ODBF, Grids, ODBFTool, ODBFCprt, ODBFMemo;

const ScaleCoef = 1.2;

type
  TMainForm = class(TForm)
    DBFTable: TDBFTable;
    DBFStructure: TDBFStructure;
    MainMenu: TMainMenu;
    NFile: TMenuItem;
    NNew: TMenuItem;
    NOpen: TMenuItem;
    N1: TMenuItem;
    NExit: TMenuItem;
    NHelp: TMenuItem;
    NAbout: TMenuItem;
    SaveDialog: TSaveDialog;
    StringGrid: TStringGrid;
    OpenDialog: TOpenDialog;
    NStructure: TMenuItem;
    NView: TMenuItem;
    NMemo: TMenuItem;
    NEdit: TMenuItem;
    NAddrecord: TMenuItem;
    NDeleterecord: TMenuItem;
    NChangememo: TMenuItem;
    N2: TMenuItem;
    NPack: TMenuItem;
    NPackmemo: TMenuItem;
    NUndeleterecord: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure NNewClick(Sender: TObject);
    procedure NOpenClick(Sender: TObject);
    procedure NStructureClick(Sender: TObject);
    procedure NExitClick(Sender: TObject);
    procedure NAboutClick(Sender: TObject);
    procedure NMemoClick(Sender: TObject);
    procedure NAddrecordClick(Sender: TObject);
    procedure NDeleterecordClick(Sender: TObject);
    procedure NUndeleterecordClick(Sender: TObject);
    procedure NChangememoClick(Sender: TObject);
    procedure NPackClick(Sender: TObject);
    procedure NPackmemoClick(Sender: TObject);
    procedure UpdateCurrentRecord;
    procedure StringGridSetEditText(Sender: TObject; ACol, ARow: Longint;
      const Value: string);
    procedure StringGridSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
    CurRow: Longint;
    Editing: Boolean;
    procedure AdjustColWidths;
    procedure ShowTable;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.AdjustColWidths;
var I, J: Integer;
begin
  with StringGrid do
    for I:=0 to ColCount-1 do
      begin
        ColWidths[I]:=DefaultColWidth;
        for J:=0 to RowCount-1 do
          if Canvas.TextWidth(Cells[I,J])>ColWidths[I]
             then ColWidths[I]:=Canvas.TextWidth(Cells[I,J]);
        ColWidths[I]:=Round(ColWidths[I]*ScaleCoef);
      end;
end;

procedure TMainForm.ShowTable;
var ADBFField: TDBFField;
    Index: Longint;
    I: Integer;
    S: String;
    N: Real;
    ADate: TDateTime;
    L: Boolean;
begin
  with StringGrid do
    begin
      ColCount:=1+DBFTable.FieldCount;
      RowCount:=1+DBFTable.RecordCount;
      Cells[0,0]:='<Del>';
      for I:=0 to DBFTable.FieldCount-1 do
        begin
          ADBFField:=DBFTable.Fields[I];
          Cells[I+1,0]:=ADBFField.FieldName;
        end;
      for Index:=1 to DBFTable.RecordCount do
        begin
          try
             DBFTable.RecNo:=Index;
          except
             AdjustColWidths;
             Caption:='DBF Table Viewer';
             if DBFTable.FileName<>''
                then Caption:=Caption+' - ['+DBFTable.FileName+']';
             raise;
          end;
          if DBFTable.Deleted
             then Cells[0,Index]:='*'
             else Cells[0,Index]:=' ';
          for I:=0 to DBFTable.FieldCount-1 do
            begin
              ADBFField:=DBFTable.Fields[I];
              case ADBFField.FieldType of
                'C': begin
                       S:=ADBFField.Text;
                       {StrPCopy(Buffer,S);
                       OemToAnsi(Buffer,Buffer);
                       S:=StrPas(Buffer);}
                     end;
                'N': begin
                       if ADBFField.Defined then
                          begin
                            N:=ADBFField.AsFloat;
                            Str(N:ADBFField.FieldLength:ADBFField.FieldDecimals,S);
                          end
                       else S:='';
                     end;
                'D': begin
                       if ADBFField.Defined then
                          begin
                            ADate:=ADBFField.AsDateTime;
                            S:=DateToStr(ADate);
                          end
                       else S:='';
                     end;
                'L': begin
                       if ADBFField.Defined then
                          begin
                            L:=ADBFField.AsBoolean;
                            case L of
                              TRUE: S:='TRUE';
                              FALSE: S:='FALSE';
                            end;
                          end
                       else S:='';
                     end;
                'M': begin
                       if ADBFField.Defined
                          then S:='<MEMO>'
                          else S:='';
                     end;
              end;
              Cells[I+1,Index]:=S;
            end;
        end;
    end;
  AdjustColWidths;
  Caption:='DBF Table Viewer';
  if DBFTable.FileName<>''
     then Caption:=Caption+' - ['+DBFTable.FileName+']';
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  CurRow:=0;
  Editing:=FALSE;
end;

procedure TMainForm.NNewClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFStructure.Clear;
  if DBFStructure.DefineFields=mrOK then
     if SaveDialog.Execute then
        begin
          DBFTable.CreateTable(SaveDialog.FileName,DBFStructure);
          DBFTable.AppendBlank;
          ShowTable;
          CurRow:=0;
          Editing:=FALSE;
        end;
end;

procedure TMainForm.NOpenClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  if OpenDialog.Execute then
     begin
       DBFTable.Use(OpenDialog.FileName);
       ShowTable;
       CurRow:=0;
       Editing:=FALSE;
     end;
end;

procedure TMainForm.NStructureClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFStructure.FieldList:=DBFTable.FieldDefs.FieldList;
  if DBFStructure.DefineFields<>mrCancel
     then MessageDlg('Changes will have no effect',mtInformation,[mbOK],0);
end;

procedure TMainForm.NExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.NAboutClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  AboutBox.ShowModal;
end;

procedure TMainForm.NMemoClick(Sender: TObject);
var ADBFField: TDBFField;
    Buffer: Array [0..255] of Char;
    FName: String;
    IH: Word;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFTable.RecNo:=StringGrid.Row;
  ADBFField:=DBFTable.Fields[StringGrid.Col-1];
  if (ADBFField.FieldType='M') then
     begin
       GetUniqueName('',Buffer);
       FName:=StrPas(Buffer);
       if ADBFField.GetMemoToFile(FName) then
          begin
            StrPCopy(Buffer,'Notepad.exe '+FName);
            IH:=WinExec(Buffer,sw_Normal);
            if IH<32
               then MessageDlg('Failed exec of Notepad',mtError,[mbOK],0);
            DeleteFile(FName);
          end
       else MessageDlg('Field value is not defined',mtWarning,[mbOK],0);
     end
  else MessageDlg('Not a memo field',mtWarning,[mbOK],0);
end;

procedure TMainForm.NAddrecordClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFTable.AppendBlank;
  ShowTable;
end;

procedure TMainForm.NDeleterecordClick(Sender: TObject);
var J: Longint;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  J:=StringGrid.Row;
  if J<=0 then exit;
  DBFTable.RecNo:=J;
  if Not DBFTable.Deleted then
     begin
       DBFTable.Delete;
       DBFTable.ReplaceRecord;   {Not necessary when AutoUpdate=TRUE}
     end;
  ShowTable;
end;

procedure TMainForm.NUndeleterecordClick(Sender: TObject);
var J: Longint;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  J:=StringGrid.Row;
  if J<=0 then exit;
  DBFTable.RecNo:=J;
  if DBFTable.Deleted then
     begin
       DBFTable.Undelete;
       DBFTable.ReplaceRecord;   {Not necessary when AutoUpdate=TRUE}
     end;
  ShowTable;
end;

procedure TMainForm.NChangememoClick(Sender: TObject);
var ADBFField: TDBFField;
    Buffer: Array [0..255] of Char;
    FName: String;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFTable.RecNo:=StringGrid.Row;
  ADBFField:=DBFTable.Fields[StringGrid.Col-1];
  if (ADBFField.FieldType='M') then
     begin
       GetUniqueName('',Buffer);
       FName:=StrPas(Buffer);
       ADBFField.GetMemoToFile(FName);
       MemoDlg:=TMemoDlg.Create(NIL);
       try
          if FileExists(FName)
             then MemoDlg.Memo.Lines.LoadFromFile(FName);
          if MemoDlg.ShowModal=mrOK then
             begin
               if (MemoDlg.Memo.Lines.Count=0) OR
                  ((MemoDlg.Memo.Lines.Count=1) AND
                  (MemoDlg.Memo.Lines[0]=''))
                  then ADBFField.ClearValue
                  else
                    begin
                      MemoDlg.Memo.Lines.SaveToFile(FName);
                      ADBFField.SetMemoFromFile(FName);
                    end;
               DBFTable.ReplaceRecord;      {Not necessary when AutoUpdate=TRUE}
               ShowTable;
             end;
       finally
          MemoDlg.Free;
          if FileExists(FName) then DeleteFile(FName);
       end;
     end
  else MessageDlg('Not a memo field',mtWarning,[mbOK],0);
end;

procedure TMainForm.NPackClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFTable.Pack;
  ShowTable;
end;

procedure TMainForm.NPackmemoClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFTable.PackMemo;
  ShowTable;
end;

procedure TMainForm.UpdateCurrentRecord;
var ACol: Longint;
    DBFField: TDBFField;
    N: Real;
    Code: Integer;
    ADate: TDateTime;
begin
  if (CurRow<=0) OR (CurRow>DBFTable.RecordCount) then exit;
  DBFTable.Go(CurRow);
  for ACol:=1 to DBFTable.FieldCount do
    begin
      DBFField:=DBFTable.Fields[ACol-1];
      if DBFField=NIL then exit;
      case DBFField.FieldType of
        'C': DBFField.AsString:=StringGrid.Cells[ACol,CurRow];
        'N': begin
               if StringGrid.Cells[ACol,CurRow]='' then
                  DBFField.ClearValue
               else
                  begin
                    Val(StringGrid.Cells[ACol,CurRow],N,Code);
                    if Code<>0 then
                       begin
                         MessageDlg(DBFField.FieldName+
                           ': New value is not a valid number',mtWarning,[mbOK],0);
                         ShowTable;
                         exit;
                       end;
                    DBFField.AsFloat:=N;
                  end;
             end;
        'D': begin
               if StringGrid.Cells[ACol,CurRow]='' then
                  DBFField.ClearValue
               else
                  begin
                    try
                       ADate:=StrToDate(StringGrid.Cells[ACol,CurRow]);
                    except
                       MessageDlg(DBFField.FieldName+
                         'New value is not a valid date',mtWarning,[mbOK],0);
                       ShowTable;
                       exit;
                    end;
                    DBFField.AsDateTime:=ADate;
                  end;
             end;
        'L': begin
               if StringGrid.Cells[ACol,CurRow]=''
                  then DBFField.ClearValue else
               if StringGrid.Cells[ACol,CurRow][1] in ['t','T','y','Y','1']
                  then DBFField.AsBoolean:=TRUE else
               if StringGrid.Cells[ACol,CurRow][1] in ['f','F','n','N','0']
                  then DBFField.AsBoolean:=FALSE else
                  begin
                    MessageDlg(DBFField.FieldName+
                      'New value is not a valid boolean',mtWarning,[mbOK],0);
                    ShowTable;
                    exit;
                  end;
             end;
      end;
    end;
  DBFTable.ReplaceRecord;   {Not necessary when AutoUpdate=TRUE}
  ShowTable;
  Editing:=FALSE;
end;

procedure TMainForm.StringGridSetEditText(Sender: TObject; ACol,
  ARow: Longint; const Value: string);
begin
  if ARow=CurRow then Editing:=TRUE;
end;

procedure TMainForm.StringGridSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
  if (CurRow<>Row) then
     begin
       if (Editing) then
          begin
            if MessageDlg('Do you wish to update the record No '+
               IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
               then UpdateCurrentRecord;
          end
       else ShowTable;
       CurRow:=Row;
       Editing:=FALSE;
       StringGrid.EditorMode:=FALSE;
     end;
  CanSelect:=TRUE;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
end;

end.

